/*
 *  tcb.c
 *
 *  ptrtd - Portable IPv6 TRT implementation
 *
 *  Copyright (C) 2001  Nathan Lutchansky
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <stdio.h>

#include "config.h"
#include "defs.h"
#include "buffer.h"
#include "tcb.h"

struct tcb *tcblist = NULL;

extern int debug;

struct tcb *tcb_new( uchar *laddr, int lport, uchar *raddr, int rport )
{
	struct tcb *t;

	t = malloc( sizeof( struct tcb ) );
	memset( t, 0, sizeof( struct tcb ) );
	t->prev = NULL;
	t->next = tcblist;
	if( tcblist ) tcblist->prev = t;
	tcblist = t;

	if( laddr ) memcpy( t->laddr, laddr, 16 );
	t->lport = lport;
	if( raddr ) memcpy( t->raddr, raddr, 16 );
	t->rport = rport;

	gettimeofday( &t->start_time, NULL );

	if( debug )
	{
		char fn[128];

		sprintf( fn, "session.%d", rport );
		if( ! ( t->debug = fopen( fn, "w" ) ) )
		{
			perror( "fopen" );
			exit( 1 );
		}
		setvbuf( t->debug, NULL, _IONBF, 0 );
	}

	return t;
}

void tcb_delete( struct tcb *t )
{
	if( t->inbuf ) rb_delete( t->inbuf );
	if( t->outbuf ) rb_delete( t->outbuf );
	if( t->debug ) fclose( t->debug );
	if( t->next ) t->next->prev = t->prev;
	if( t->prev ) t->prev->next = t->next;
	if( t == tcblist ) tcblist = t->next;

	free( t );
}

struct tcb *tcb_find( uchar *laddr, int lport, uchar *raddr, int rport )
{
	struct tcb *t;
	uchar zaddr[16] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };

	if( ! laddr ) laddr = zaddr;
	if( ! raddr ) raddr = zaddr;
	for( t = tcblist; t; t = t->next )
	{
		if( 	! memcmp( t->raddr, raddr, 16 )
			&& rport == t->rport
			&& ! memcmp( t->laddr, laddr, 16 )
			&& lport == t->lport )
		{
			return t;
		}
	}
	if( raddr != zaddr ) return tcb_find( laddr, lport, NULL, 0 );
	if( laddr != zaddr ) return tcb_find( NULL, lport, NULL, 0 );
	if( lport > 0 ) return tcb_find( NULL, 0, NULL, 0 );
	return NULL;
}
