/*
 *  util.c
 *
 *  ptrtd - Portable IPv6 TRT implementation
 *
 *  Copyright (C) 2001  Nathan Lutchansky
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "config.h"
#include "util.h"

void dump_packet( char *title, struct pbuf *pkt )
{
	int j, len = pkt->dlen;
	char b[17];
	uchar *p = pkt->d;
	printf( "Packet: %s\n", title );

	for( j = 0; j < len; ++j )
	{
		if( j % 16 == 0 ) printf( "%08x  ", j );

		printf( "%02x ", p[j] );
		b[j % 16] = isprint( p[j] ) ? p[j] : '.';

		if( j % 16 == 15 || j == len - 1 )
		{
			b[j % 16 + 1] = 0;
			if( j == len - 1 )
			{
				j = j % 16;
				while( ++j < 16 ) printf( "   " );
				j = len;
			}
			printf( " %s\n", b );
		}
	}
}

int ip6tostr( char *str, uchar *addr )
{
	sprintf( str, "%x:%x:%x:%x:%x:%x:%x:%x",
			GET_16( addr + 0 ), GET_16( addr + 2 ),
			GET_16( addr + 4 ), GET_16( addr + 6 ),
			GET_16( addr + 8 ), GET_16( addr + 10 ),
			GET_16( addr + 12 ), GET_16( addr + 14 ) );
	return 0;
}

int strtoip6( uchar *addr, char *str )
{
	char *c;
	unsigned int tmp;
	int i = 0, skip = 0;
	char *zero = "0";
	char *g[8] = { zero, zero, zero, zero, zero, zero, zero, zero };

	for( c = str + strlen( str ); c != str; --c )
		if( c[0] == ':' )
		{
			++skip;
			if( c[-1] == ':' ) break;
		}

	if( str[0] == ':' ) c = str;
	else c = str - 1;
	for( ; c; c = strchr( c, ':' ) )
	{
		++c;
		if( *c == ':' )
		{
			i = 8 - skip;
			++c;
			if( ! *c ) break;
		}
		g[i] = c;
		++i;
	}
	for( i = 0; i < 8; ++i )
	{
		sscanf( g[i], "%x", &tmp );
		PUT_16( addr + 2 * i, tmp );
	}

	return 0;
}

int make_cksum( uchar *p, int len )
{
	int i;
	int c = GET_16( p + 4 ) + p[6];

	for( i = 0x8; i < len; ++i )
		c += (i % 2) ? p[i] : (p[i] << 8);
	while( c > 0xffff )
		c = (c >> 16) + (c & 0xffff);
	return c;
}
